#import "ONSLController.h"
#import "ONSLGameManager.h"
#import "ONSLGameInfo.h"
#import "DragImageView.h"
#import "ONSLFontManager.h"

@implementation ONSLController

- (id)init
{
	[super init];
	mgr = [[ONSLGameManager alloc] initWithDelegate:self];
	fontMgr = [[ONSLFontManager alloc] init];
	quitOK = NO;
	return self;
}

- (void)dealloc
{
	[mgr release];
	[fontMgr release];
	[super dealloc];
}

- (IBAction)addGame:(id)sender
{
	[o_prefGameIcon setImage:[NSImage imageNamed:@"NSApplicationIcon"]];
	[o_prefGameIcon clearPath];
	[o_prefGameName setStringValue:@"New Game"];
	[o_prefGameDirectory setStringValue:@""];
	[o_prefGameOption setStringValue:@""];
	[o_fullscreenMode setState:NSOffState];
	[o_varEditMode setState:NSOffState];
	[self createFontMenu];
	[o_prefGameFont selectItemAtIndex:0];
	[NSApp beginSheet:o_gameEditPane
		modalForWindow:o_mainWindow
		modalDelegate:self
		didEndSelector:@selector(sheetDidEnd:returnCode:contextInfo:)
		contextInfo:@"AddGame"];
}

- (IBAction)editGame:(id)sender
{
	ONSLGameInfo *gi = [mgr gameInfoAtIndex:[o_gameListTable selectedRow]];
	[o_prefGameName setStringValue:[gi title]];
	[o_prefGameDirectory setStringValue:[gi path]];
	[o_prefGameOption setStringValue:[gi otherArgs]];
	if([gi fullscreenMode]) [o_fullscreenMode setState:NSOnState];
	else [o_fullscreenMode setState:NSOffState];
	if([gi varEditMode]) [o_varEditMode setState:NSOnState];
	else [o_varEditMode setState:NSOffState];
	[self createFontMenu];
	if([fontMgr fontNamed: [gi fontName]]) [o_prefGameFont selectItemWithTitle:[gi fontName]];
	else [o_prefGameFont selectItemAtIndex:0];
	if([[gi imagePath] isEqualToString:@""]) {
		[o_prefGameIcon setImage:[NSImage imageNamed:@"NSApplicationIcon"]];
		[o_prefGameIcon clearPath];
	}
	else {
		[o_prefGameIcon setImgPath:[gi imagePath]];
		if([[NSFileManager defaultManager] isReadableFileAtPath:[gi imagePath]])
			[o_prefGameIcon setImage:[[[NSImage alloc] initByReferencingFile:[gi imagePath]] autorelease]];
		else [o_prefGameIcon setImage:[NSImage imageNamed:@"NSApplicationIcon"]];
	}
	[NSApp beginSheet:o_gameEditPane
		modalForWindow:o_mainWindow
		modalDelegate:self
		didEndSelector:@selector(sheetDidEnd:returnCode:contextInfo:)
		contextInfo:@"EditGame"];
	
}

- (IBAction)endEdit:(id)sender
{
	if([sender tag] == 0) {
		if([[o_prefGameName stringValue] isEqualToString:@""]) {
			NSRunCriticalAlertPanel(LS(@"error"), LS(@"name is empty"), LS(@"Cancel"), nil, nil);
			return;
		}
		if([[o_prefGameDirectory stringValue] isEqualToString:@""]) {
			NSRunCriticalAlertPanel(LS(@"error"), LS(@"path is empty"), LS(@"Cancel"), nil, nil);
			return;
		}
	}
	[NSApp endSheet:o_gameEditPane returnCode:[sender tag]];
	[o_gameEditPane makeFirstResponder:o_prefGameName];
	[o_gameEditPane close];
}

- (IBAction)removeGame:(id)sender
{
	NSBeginAlertSheet(LS(@"remove"), @"OK", nil, @"Cancel", o_mainWindow, self, @selector(sheetDidEnd:returnCode:contextInfo:), nil, @"RemoveGame", [NSString stringWithFormat:LS(@"remove ok?"),[[mgr gameInfoAtIndex:[o_gameListTable selectedRow]] title]]);
}

- (IBAction)launchGame:(id)sender
{
	id game = [mgr gameInfoAtIndex:[o_gameListTable selectedRow]];
	if([game isPlaying]) {
		NSBeginCriticalAlertSheet(LS(@"quit"), @"OK", nil, @"Cancel", o_mainWindow, self, @selector(sheetDidEnd:returnCode:contextInfo:), nil, @"Terminate", [NSString stringWithFormat:LS(@"%@: terminate ok?"), [game title]]);
		return;
	}
	else [game execute];
	[self updateTable];
}

- (IBAction)chooseGameDirectory:(id)sender
{
	NSOpenPanel *op = [NSOpenPanel openPanel];
	[op setCanChooseDirectories:YES];
	[op setCanChooseFiles:NO];
	[op setAllowsMultipleSelection:NO];
	
	int ret = [op runModalForDirectory:nil file:nil types:nil];
	if(ret != NSOKButton) return;
	
	[o_prefGameDirectory setStringValue:[op filename]];
}

- (IBAction)removeIcon:(id)sender
{
	[o_prefGameIcon clearPath];
	[o_prefGameIcon setImage:[NSImage imageNamed:@"NSApplicationIcon"]];
}

- (IBAction)addFont:(id)sender
{
	[o_prefFontName setStringValue:@"New Font"];
	[o_prefFontPath setStringValue:@""];
	[NSApp beginSheet:o_fontEditPane
		modalForWindow:o_prefPane
		modalDelegate:self
		didEndSelector:@selector(sheetDidEnd:returnCode:contextInfo:)
		contextInfo:@"AddFont"];
}

- (IBAction)editFont:(id)sender
{
	NSString *name = [fontMgr fontNameAtIndex: [o_prefFontTable selectedRow]];
	[o_prefFontName setStringValue:name];
	[o_prefFontPath setStringValue:[fontMgr fontNamed:name]];
	[NSApp beginSheet:o_fontEditPane
		modalForWindow:o_prefPane
		modalDelegate:self
		didEndSelector:@selector(sheetDidEnd:returnCode:contextInfo:)
		contextInfo:@"EditFont"];
}

- (IBAction)removeFont:(id)sender
{
	NSString *name = [fontMgr fontNameAtIndex:[o_prefFontTable selectedRow]];
	[fontMgr removeFont:name];
	[mgr fontRemoved:name];
	[o_prefFontTable reloadData];
	[o_prefGameFont removeItemWithTitle:name];
}

- (IBAction)chooseFont:(id)sender
{
	NSOpenPanel *op = [NSOpenPanel openPanel];
	[op setCanChooseDirectories:NO];
	[op setCanChooseFiles:YES];
	[op setAllowsMultipleSelection:NO];
	
	int ret = [op runModalForDirectory:nil file:nil types:[NSArray arrayWithObjects:@"ttf",@"ttc",@"otf",@"dfont",nil]];
	if(ret != NSOKButton) return;
	
	[o_prefFontPath setStringValue:[op filename]];
}

- (IBAction)endEditFont:(id)sender
{
	if([sender tag] == 0) {
		if([[o_prefFontName stringValue] isEqualToString:@""]) {
			NSRunCriticalAlertPanel(LS(@"error"), LS(@"font name is empty"), LS(@"Cancel"), nil, nil);
			return;
		}
		if([[o_prefFontPath stringValue] isEqualToString:@""]) {
			NSRunCriticalAlertPanel(LS(@"error"), LS(@"font path is empty"), LS(@"Cancel"), nil, nil);
			return;
		}
		if([fontMgr fontNamed:[o_prefFontName stringValue]]) {
			NSRunCriticalAlertPanel(LS(@"error"), LS(@"font name is wrong"), LS(@"Cancel"), nil, nil);
			return;
		}
	}
	[NSApp endSheet:o_fontEditPane returnCode:[sender tag]];
	[o_fontEditPane makeFirstResponder:o_prefFontName];
	[o_fontEditPane close];
	[o_prefPane makeKeyAndOrderFront: self];
}

- (IBAction)doubleClicked:(id)sender
{
	if([sender tag] == 0) {
		if([o_gameListTable numberOfSelectedRows] == 0) return;
		else [[mgr gameInfoAtIndex:[o_gameListTable selectedRow]] execute];
		[self updateTable];
	}
	else if([sender tag] == 1) {
		if([o_prefFontTable numberOfSelectedRows] == 0) return;
		else [o_prefFontEdit performClick:nil];
	}
}

- (void)addLogText:(NSString *)str
{
	[o_logView selectAll:nil];
	[o_logView setSelectedRange:NSMakeRange([o_logView selectedRange].length, 0)];
	[o_logView setEditable:YES];
	NSMutableAttributedString *attStr = [[NSMutableAttributedString alloc] initWithString:str];
	[attStr setAttributes:[NSDictionary dictionaryWithObject:[NSFont systemFontOfSize:10] forKey:NSFontAttributeName] range:NSMakeRange(0,[str length])];
	[o_logView insertText:attStr];
	[o_logView setEditable:NO];
	[attStr release];
}

- (void)updateTable
{
	[o_gameListTable reloadData];
	[self tableViewSelectionDidChange:[NSNotification notificationWithName:nil object:o_gameListTable]];
}

- (void)savePrefs
{
	NSUserDefaults *pref = [NSUserDefaults standardUserDefaults];
	[pref setObject:[mgr saveGameList] forKey:@"GameList"];
	[pref setObject:[fontMgr saveFontList] forKey:@"FontList"];
	[pref setInteger:[[o_selectOnscripter selectedCell] tag] forKey:@"ONScripter"];
	[pref setObject:[o_executePath stringValue] forKey:@"ExecutePath"];
	[pref setInteger:[o_terminateOnExit state] forKey:@"TerminateOnExit"];
	[pref synchronize];
}

- (void)loadPrefs
{
	NSUserDefaults *pref = [NSUserDefaults standardUserDefaults];
	id obj;
	if(obj=[pref objectForKey:@"GameList"]) {
		[mgr loadGameList:obj];
		[o_gameListTable reloadData];
		[mgr redrawGameInfoAtTable:o_gameListTable];
	}
	if(obj=[pref objectForKey:@"FontList"]) {
		[fontMgr loadFontList:obj];
		[o_prefFontTable reloadData];
	}
	if(obj=[pref objectForKey:@"ONScripter"]) {
		[o_selectOnscripter selectCellWithTag:[obj intValue]];
	}
	if(obj=[pref objectForKey:@"ExecutePath"]) {
		[o_executePath setStringValue:obj];
	}
	if(obj=[pref objectForKey:@"TerminateOnExit"]) {
		[o_terminateOnExit setState:[obj intValue]];
	}
}

- (void)createFontMenu
{
	[o_prefGameFont removeAllItems];
	[o_prefGameFont addItemsWithTitles:[fontMgr fontNames]];
}

- (id)fontManager
{
	return fontMgr;
}

- (NSString *)executePath
{
	if([[o_selectOnscripter selectedCell] tag] == 1) {
		if([[NSFileManager defaultManager] isExecutableFileAtPath:[o_executePath stringValue]])
			return [o_executePath stringValue];
		else {
			[self addLogText:LS(@"onscripter not found")];
			return [[NSBundle mainBundle] pathForResource:@"onscripter" ofType:nil inDirectory:nil];
		}
	}
	else return [[NSBundle mainBundle] pathForResource:@"onscripter" ofType:nil inDirectory:nil];
}



- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	ONSLGameInfo *gi;
	if([(NSString *)contextInfo isEqualToString:@"AddGame"]) {
		switch(returnCode) {
			case 0:
				gi = [[ONSLGameInfo alloc] initWithDelegate:self];
				[gi setTitle:[o_prefGameName stringValue]];
				[gi setPath:[o_prefGameDirectory stringValue]];
				[gi setFontName:[[o_prefGameFont selectedItem] title]];
				[gi setImagePath:[o_prefGameIcon imgPath]];
				[gi setOtherArgs:[o_prefGameOption stringValue]];
				if([o_fullscreenMode state] == NSOnState) [gi setFullscreenMode:YES];
				else [gi setFullscreenMode:NO];
				if([o_varEditMode state] == NSOnState) [gi setVarEditMode:YES];
				else [gi setVarEditMode:NO];
				break;
			default:
				return;
		}
		[mgr addGameInfo:gi];
		[o_gameListTable reloadData];
		[gi addToTable:o_gameListTable row:[o_gameListTable numberOfRows]-1];
		[gi release];
	}
	else if([(NSString *)contextInfo isEqualToString:@"EditGame"]) {
		switch(returnCode) {
			case 0:
				gi = [mgr gameInfoAtIndex:[o_gameListTable selectedRow]];
				[gi setTitle:[o_prefGameName stringValue]];
				[gi setPath:[o_prefGameDirectory stringValue]];
				[gi setFontName:[[o_prefGameFont selectedItem] title]];
				[gi setImagePath:[o_prefGameIcon imgPath]];
				[gi setOtherArgs:[o_prefGameOption stringValue]];
				if([o_fullscreenMode state] == NSOnState) [gi setFullscreenMode:YES];
				else [gi setFullscreenMode:NO];
				if([o_varEditMode state] == NSOnState) [gi setVarEditMode:YES];
				else [gi setVarEditMode:NO];
				break;
			default:
				return;
		}
		[o_gameListTable reloadData];
	}
	else if([(NSString *)contextInfo isEqualToString:@"Terminate"]) {
		switch(returnCode) {
			case NSAlertDefaultReturn:
				gi = [mgr gameInfoAtIndex:[o_gameListTable selectedRow]];
				[gi terminate];
				break;
			default:
				return;
		}
	}
	else if([(NSString *)contextInfo isEqualToString:@"Quit"]) {
		switch(returnCode) {
			case NSAlertDefaultReturn:
				if([o_terminateOnExit state] == NSOnState) [mgr terminateAllGames];
				quitOK = YES;
				[NSApp terminate:nil];
				break;
			default:
				return;
		}
	}
	else if([(NSString *)contextInfo isEqualToString:@"AddFont"]) {
		switch(returnCode) {
			case 0:
				[fontMgr addNewFont:[o_prefFontName stringValue] path:[o_prefFontPath stringValue]];
				break;
			default:
				return;
		}
		[o_prefFontTable reloadData];
	}
	else if([(NSString *)contextInfo isEqualToString:@"EditFont"]) {
		switch(returnCode) {
			NSString *oldName;
			case 0:
				oldName = [fontMgr fontNameAtIndex: [o_prefFontTable selectedRow]];
				[fontMgr editFont:oldName newName:[o_prefFontName stringValue] newPath:[o_prefFontPath stringValue]];
				[mgr fontNameHasChangedFrom:oldName to:[o_prefFontName stringValue]];
				[[o_prefGameFont itemWithTitle:oldName] setTitle:[o_prefFontName stringValue]];
				break;
			default:
				return;
		}
		[o_prefFontTable reloadData];
	}
	else if([(NSString *)contextInfo isEqualToString:@"RemoveGame"]) {
		switch(returnCode) {
			case NSAlertDefaultReturn:
				[mgr removeGameInfoAtIndex:[o_gameListTable selectedRow]];
				[o_gameListTable reloadData];
				[mgr redrawGameInfoAtTable:o_gameListTable];
				break;
			default:
				return;
		}
	}
			
}

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
	if([tableView tag] == 0) return [mgr numberOfGames];
	else if([tableView tag] == 1) return [fontMgr numberOfFonts];
	return 0;
}

- (id)tableView:(NSTableView *)tableView
		objectValueForTableColumn:(NSTableColumn *)tableColumn
		row:(int)row
{
	if([tableView tag] == 0) {
		if([[tableColumn identifier] isEqual:@"Image"]) {
			NSImage *img;
			NSString *imgPath = [[mgr gameInfoAtIndex:row] imagePath];
			if([imgPath isEqualToString:@""]) img = [NSImage imageNamed:@"NSApplicationIcon"];
			else {
				if([[NSFileManager defaultManager] isReadableFileAtPath:imgPath])
					img = [[[NSImage alloc] initByReferencingFile:imgPath] autorelease];
				else img = [NSImage imageNamed:@"NSApplicationIcon"];
			}
			
			return img;
		}
		else if([[tableColumn identifier] isEqual:@"State"]) {
			if([[mgr gameInfoAtIndex:row] isPlaying]) return [NSImage imageNamed:@"playing2"];
			else return nil;
		}
		return nil;
	}
	else if([tableView tag] == 1) {
		if([[tableColumn identifier] isEqual:@"Name"]) {
			return [fontMgr fontNameAtIndex:row];
		}
		else if([[tableColumn identifier] isEqual:@"Path"]) {
			return [fontMgr fontNamed:[fontMgr fontNameAtIndex:row]];
		}
		return nil;
	}
	return nil;
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	if([[aNotification object] tag] == 0) {
		if([o_gameListTable numberOfSelectedRows] == 0) {
			[o_launchButton setTitle:LS(@"launch")];
			[o_editButton setEnabled:NO];
			[o_launchButton setEnabled:NO];
			[o_removeButton setEnabled:NO];
		}
		else {
			if([[mgr gameInfoAtIndex:[o_gameListTable selectedRow]] isPlaying]) {
				[o_launchButton setTitle:LS(@"stop")];
				[o_removeButton setEnabled:NO];
			}
			else {
				[o_launchButton setTitle:LS(@"launch")];
				[o_removeButton setEnabled:YES];
			}
			[o_editButton setEnabled:YES];
			[o_launchButton setEnabled:YES];
		}
	}
	else if([[aNotification object] tag] == 1) {
		if([o_prefFontTable numberOfSelectedRows] == 0) {
			[o_prefFontEdit setEnabled:NO];
			[o_prefFontRemove setEnabled:NO];
		}
		else if([o_prefFontTable selectedRow] < 4) {
			[o_prefFontEdit setEnabled:NO];
			[o_prefFontRemove setEnabled:NO];
		}
		else {
			[o_prefFontEdit setEnabled:YES];
			[o_prefFontRemove setEnabled:YES];
		}
	}
}

- (void)windowDidResize:(NSNotification *)aNotification
{
	if(![mgr numberOfGames]) return;
	[mgr setWidthOfGameInfo: [o_gameListTable frameOfCellAtColumn:1 row:0].size.width];
}

- (void)applicationDidFinishLaunching: (NSNotification *)notification
{
	[[[o_gameListTable tableColumns] objectAtIndex:0] setDataCell:[[NSImageCell alloc]init]];
	[[[o_gameListTable tableColumns] objectAtIndex:2] setDataCell:[[NSImageCell alloc]init]];
	[self loadPrefs];
	
	[o_gameListTable setDoubleAction:@selector(doubleClicked:)];
	[o_prefFontTable setDoubleAction:@selector(doubleClicked:)];
	[o_mainWindow makeKeyAndOrderFront: self];
	NSGraphicsContext *gc = [NSGraphicsContext currentContext];
	[gc setImageInterpolation:NSImageInterpolationHigh];
	[o_gameListTable reloadData];
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
	if(!quitOK && ![mgr terminateOK]) {
		NSBeginCriticalAlertSheet(LS(@"quit ONSLauncher"), @"OK", nil, @"Cancel", o_mainWindow, self, @selector(sheetDidEnd:returnCode:contextInfo:), nil, @"Quit", LS(@"quit ok?"));
		return NO;
	}
	
	[self savePrefs];
	return YES;
}

- (BOOL)windowShouldClose:(id)sender
{
	[NSApp terminate:nil];
	return NO;
}

@end
