//
//  ONSLGameInfo.m
//  ONSLauncher
//
//  Created by tmkk on Wed Mar 08 2006.
//  Copyright (c) 2006 tmkk. All rights reserved.
//

#import "ONSLGameInfo.h"
#import "ONSLController.h"
#import "ONSLFontManager.h"
#import <unistd.h>

@implementation ONSLGameInfo

- (id)init
{
	[super init];
	title = nil;
	path = nil;
	fontName = nil;
	imagePath = nil;
	task = nil;
	delegate = nil;
	otherArgs = nil;
	titleField = [[NSTextField alloc] init];
	pathField = [[NSTextField alloc] init];
	[titleField setBordered:NO];
	[titleField setEditable:NO];
	[[titleField cell] setWraps:NO];
	[titleField setBackgroundColor:[NSColor controlColor]];
	[pathField setBordered:NO];
	[pathField setEditable:NO];
	[[pathField cell] setWraps:NO];
	[pathField setBackgroundColor:[NSColor controlColor]];
	isPlaying = NO;
	fullscreenMode = NO;
	varEditMode = NO;
	return self;
}

- (id)initWithDelegate:(id)del
{
	[self init];
	delegate = [del retain];
	return self;
}

- (void)dealloc
{
	if(title) [title release];
	if(path) [path release];
	if(fontName) [fontName release];
	if(imagePath) [imagePath release];
	if(delegate) [delegate release];
	if(otherArgs) [otherArgs release];
	if(task) [task release];
	[titleField release];
	[pathField release];
	[super dealloc];
}

- (void)setTitle:(NSString *)str
{
	if(title) [title release];
	title = [str retain];
	NSMutableAttributedString *attStr = [[NSMutableAttributedString alloc] initWithString:str];
	[attStr setAttributes:[NSDictionary dictionaryWithObject:[NSFont boldSystemFontOfSize:13] forKey:NSFontAttributeName] range:NSMakeRange(0,[str length])];
	[titleField setAttributedStringValue:attStr];
	[attStr release];
}

- (void)setPath:(NSString *)str
{
	if(path) [path release];
	path = [str retain];
	NSMutableAttributedString *attStr = [[NSMutableAttributedString alloc] initWithString:str];
	[attStr setAttributes:[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:[NSFont systemFontOfSize:10],[NSColor lightGrayColor],nil] forKeys:[NSArray arrayWithObjects:NSFontAttributeName,NSForegroundColorAttributeName,nil]] range:NSMakeRange(0,[str length])];
	[pathField setAttributedStringValue:attStr];
	[attStr release];
}

- (void)setFontName:(NSString *)str
{
	if(fontName) [fontName release];
	fontName = [str retain];
}

- (void)setImagePath:(NSString *)str
{
	if(imagePath) [imagePath release];
	imagePath = [str retain];
}

- (void)setOtherArgs:(NSString *)str
{
	if(otherArgs) [otherArgs release];
	otherArgs = [str retain];
}

- (NSString *)title
{
	return title;
}

- (NSString *)path
{
	return path;
}

- (NSString *)fontName
{
	return fontName;
}

- (NSString *)imagePath
{
	return imagePath;
}

- (NSString *)otherArgs
{
	return otherArgs;
}

- (void)addToTable:(id)table row:(int)row
{
	NSRect frame1 = [table frameOfCellAtColumn:1 row:row];
	frame1.origin.y += 1;
	frame1.size.height = frame1.size.height/2-1;
	NSRect frame2 = frame1;
	frame2.origin.y += frame2.size.height+2;
	frame2.size.height = frame2.size.height-6;
	[titleField setFrame:frame1];
	[pathField setFrame:frame2];
	[table addSubview:titleField];
	[table addSubview:pathField];
}

- (void)removeFromTable
{
	[titleField removeFromSuperview];
	[pathField removeFromSuperview];
}

- (void)setFrameWidth:(int)width
{
	NSRect frame1 = [titleField frame];
	NSRect frame2 = [pathField frame];
	frame1.size.width = width;
	frame2.size.width = width;
	[titleField setFrame:frame1];
	[pathField setFrame:frame2];
}

- (BOOL)isPlaying
{
	return isPlaying;
}

- (void)setPlaying:(BOOL)flag
{
	isPlaying = flag;
}

- (void)execute
{
	if(isPlaying) return;
	BOOL flag1,flag2;
	flag1 = [[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:&flag2];
	if(!(flag1 && flag2)) {
		[delegate addLogText:LS(@"directory not found")];
		return;
	}
	[self clearTask];
	task = [[NSTask alloc] init];
	id pipe = [NSPipe pipe];
	[task setStandardOutput: pipe];
	[task setStandardError: pipe];
	[task setLaunchPath: [delegate executePath]];
	NSString *fontPath = [[delegate fontManager] fontNamed:fontName];
	if(!fontPath) {
		fontPath = [[delegate fontManager] fontNamed:[[delegate fontManager] fontNameAtIndex:0]];
		[delegate addLogText:LS(@"font not found")];
	}
	else if(![[NSFileManager defaultManager] isReadableFileAtPath:fontPath]) {
		fontPath = [[delegate fontManager] fontNamed:[[delegate fontManager] fontNameAtIndex:0]];
		[delegate addLogText:LS(@"font not found")];
	}
	NSMutableArray *args = [NSMutableArray arrayWithObjects:@"-f", fontPath, @"-r", path, nil];
	if(fullscreenMode) [args addObject:@"--fullscreen"];
	else [args addObject:@"--window"];
	if(varEditMode) [args addObject:@"--edit"];
	if(![otherArgs isEqualToString:@""]) [args addObjectsFromArray:[otherArgs componentsSeparatedByString:@" "]];
	[args removeObject:@""];
	[task setArguments: args];
	
	/*[[NSNotificationCenter defaultCenter] addObserver:self 
        selector:@selector(getData:) 
        name: @"NSFileHandleReadCompletionNotification"
        object: [pipe fileHandleForReading]];*/
	[[NSNotificationCenter defaultCenter] addObserver:self 
        selector:@selector(taskDidEnd:) 
        name: @"NSTaskDidTerminateNotification" 
        object: nil];
	
	//[[pipe fileHandleForReading] readInBackgroundAndNotify];
	
	isPlaying = YES;
	[NSThread detachNewThreadSelector:@selector(watchOutput) toTarget:self withObject:nil];
	[task launch];
}

- (void)terminate
{
	if(!isPlaying) return;
	[task terminate];
}

- (void)clearTask
{
	if(task) [task release];
	task = nil;
}

- (void)setFullscreenMode:(BOOL)flag
{
	fullscreenMode = flag;
}

- (void)setVarEditMode:(BOOL)flag
{
	varEditMode = flag;
}

- (BOOL)fullscreenMode
{
	return fullscreenMode;
}

- (BOOL)varEditMode
{
	return varEditMode;
}


- (void)taskDidEnd: (NSNotification *)aNotification
{
	if([task isRunning]) return;
	//[[[task standardOutput] fileHandleForReading] closeFile];
	//[[[task standardError] fileHandleForReading] closeFile];
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[self clearTask];
	isPlaying = NO;
	[delegate updateTable];
}

- (void)getData: (NSNotification *)aNotification
{
	NSData *data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];
	if ([data length]) {
		NSString *str = [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease];
		[delegate addLogText:[NSString stringWithFormat:@"%@:\t%@",title,str]];
	}
	if([task isRunning]) [[aNotification object] readInBackgroundAndNotify];
}

- (void)watchOutput
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	NSFileHandle *handle = [[[task standardOutput] fileHandleForReading] retain];
	unsigned char *buffer = (unsigned char*)malloc(1024);
	int p=0;
	
	while(isPlaying) {
		NSAutoreleasePool *pool2 = [[NSAutoreleasePool alloc] init];
		
		NSData *data = [handle availableData];
		if(data && [data length] > 0) {
			int i;
			unsigned char *ptr = (unsigned char *)[data bytes];
			for(i=0;i<[data length];i++) {
				buffer[p++] = ptr[i];
				if(ptr[i] == '\n' || p == 1024) {
					buffer[p-1] = 0;
					NSString *str = [NSString stringWithCString:buffer];
					[delegate addLogText:[NSString stringWithFormat:@"%@:\t%@\n",title,str]];
					p = 0;
				}
			}
		}
		
		[pool2 release];
	}
	free(buffer);
	[handle release];
	[pool release];
}

@end
